/*
 *  Copyright (C) 2013  Thadeu Lima de Souza Cascardo <cascardo@cascardo.info>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "friend.h"
#include <string.h>
#include <glib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

static int connect_friend(struct sockaddr **saddr, char *address, char *port)
{
	struct addrinfo *addresses;
	struct addrinfo *addr;
	struct addrinfo hint;
	int r;
	memset(&hint, 0, sizeof(hint));
	hint.ai_family = AF_UNSPEC;
	hint.ai_socktype = SOCK_STREAM;
	hint.ai_protocol = IPPROTO_TCP;
	hint.ai_flags = AI_ADDRCONFIG;
	r = getaddrinfo(address, port, &hint, &addresses);
	if (r) {
		return r;
	}
	if (addresses != NULL) {
		*saddr = g_malloc(addresses->ai_addrlen);
		if (!*saddr) {
			r = -1;
		} else {
			memcpy(*saddr, addresses->ai_addr, addresses->ai_addrlen);
		}
	}
	freeaddrinfo(addresses);
	if (r == -1)
		return EAI_SYSTEM;
	return 0;
}

struct friend {
	char *name;
	char *address;
	char *port;
	struct sockaddr *saddr;
};

struct cache {
	GList *friends;
};

int create_cache(struct cache **cache)
{
	*cache = g_slice_new0(struct cache);
	(*cache)->friends = NULL;
	return 0;
}

static void destroy_friend(gpointer data)
{
	struct friend *friend = data;
	g_free(friend->name);
	g_free(friend->address);
	g_free(friend->port);
	g_free(friend->saddr);
	g_slice_free(struct friend, friend);
}

int destroy_cache(struct cache *cache)
{
	if (cache->friends)
		g_list_free_full(cache->friends, destroy_friend);
	g_slice_free(struct cache, cache);
}

int cache_add_friend(struct cache *cache, char *name, char *address, char *port)
{
	struct friend *friend;
	friend = g_slice_new0(struct friend);
	friend->name = g_strdup(name);
	friend->address = g_strdup(address);
	friend->port = g_strdup(port);
	connect_friend(&friend->saddr, friend->address, friend->port);
	g_list_append(cache->friends, friend);
	return 0;
}

int load_cache(struct cache *cache, char *fname)
{
	GKeyFile *file;
	gchar **groups;
	gchar **group;
	file = g_key_file_new();
	g_key_file_load_from_file(file, fname, G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, NULL);
	groups = g_key_file_get_groups(file, NULL);
	for (group = groups; *group != NULL; group++) {
		gchar *name;
		gchar *address;
		gchar *port;
		name = g_key_file_get_value(file, *group, "name", NULL);
		address = g_key_file_get_value(file, *group, "address", NULL);
		port = g_key_file_get_value(file, *group, "port", NULL);
		cache_add_friend(cache, name, address, port);
		g_free(name);
		g_free(address);
		g_free(port);
	}
	g_strfreev(groups);
	g_key_file_free(file);
	return 0;
}

int store_cache(struct cache *cache, char *fname)
{
	GKeyFile *file;
	GList *f;
	file = g_key_file_new();
	g_key_file_load_from_file(file, fname, G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, NULL);
	for (f = g_list_first(cache->friends); f != NULL; f = g_list_next(f)) {
		struct friend *friend = f->data;
		g_key_file_set_value(file, friend->name, "name", friend->name);
		g_key_file_set_value(file, friend->name, "address", friend->address);
		g_key_file_set_value(file, friend->name, "port", friend->port);
	}
	g_key_file_free(file);
	return 0;
}
